@@ -20,7 +20,7 @@ except NameError:
 
 class CLIParams(
         namedtuple("CLIParams",
-                   "connect_timeout run_once run_from_stdin sync_connect hosts")):
+                   "connect_timeout run_once run_from_stdin sync_connect hosts readonly")):
     """
     This defines the running params for a CLI() object. If you'd like to do parameters processing
     from some other point you'll need to fill up an instance of this class and pass it to
@@ -56,6 +56,10 @@ def get_params():
                         action="store_true",
                         default=False,
                         help="Connect syncronously.")
+    parser.add_argument("--readonly",
+                        action="store_true",
+                        default=False,
+                        help="Enable readonly.")
     parser.add_argument("hosts",
                         nargs="*",
                         help="ZK hosts to connect")
@@ -65,7 +69,8 @@ def get_params():
         params.run_once,
         params.run_from_stdin,
         params.sync_connect,
-        params.hosts)
+        params.hosts,
+        params.readonly)
 
 
 class StateTransition(Exception):
@@ -78,6 +83,22 @@ def sigusr_handler(*_):
     raise StateTransition()
 
 
+def set_unbuffered_mode():
+    """
+    make output unbuffered
+    """
+    class Unbuffered(object):
+        def __init__(self, stream):
+            self.stream = stream
+        def write(self, data):
+            self.stream.write(data)
+            self.stream.flush()
+        def __getattr__(self, attr):
+            return getattr(self.stream, attr)
+
+    sys.stdout = Unbuffered(sys.stdout)
+
+
 class CLI(object):
     """ the REPL """
 
@@ -90,10 +111,16 @@ class CLI(object):
 
         interactive = params.run_once == "" and not params.run_from_stdin
         async = False if params.sync_connect or not interactive else True
+
+        if not interactive:
+            set_unbuffered_mode()
+
         shell = Shell(params.hosts,
                       params.connect_timeout,
                       setup_readline=interactive,
-                      async=async)
+                      output=sys.stdout,
+                      async=async,
+                      read_only=params.readonly)
 
         if not interactive:
             rc = 0
